/*
 * Application.java
 *
 * ECHA is providing program source code to facilitate the preparation of
 * pre-registration XML files. The code shall serve as example and
 * registrants are under no circumstances obliged to make use of this service
 * for submitting data to ECHA. In no event shall ECHA be liable for any
 * damages, direct, incidental or consequential (including through negligence
 * or otherwise) which may result from the use of the data provided, even if
 * ECHA has been advised of the possibility of such damages.
 * The users must assume the entire risk of using this source code.
 */

package eu.europa.echa.reach.preregistration.examples.preregcheck;

import java.awt.Color;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

/**
 *
 * @author Csk Gbor
 */
public class Application {
    private static Properties preferences;
    
    private static final Logger logger = Logger.getLogger(Application.class);
    
    public Application() {
    }
    
    public static void main(String[] args) {
        //configure default logging
        BasicConfigurator.configure();
        Logger.getRootLogger().setLevel(Level.WARN);
        
        logger.debug("Application Start");
        
        setPreferredParserProperties();
        
        if (args.length == 0) {
            // without arguments start GUI
            
            java.awt.EventQueue.invokeLater(new Runnable() {
                public void run() {
                    new MainFrame().setVisible(true);
                }
            });
            
        } else {
            Validator validator = new Validator();
            validator.setMessageWriter(new Validator.MessageWriter() {
                public void write(final String message, final Color color) {
                    System.out.print(message);
                }
            });
            validator.validate(args);
        } 
    }
    
    public static Properties getPreferences() {
        if (preferences == null) {
            loadPreferences();
        }
        return preferences;
    }
    
    private static void loadPreferences() {
        preferences = new Properties();
        File preferencesFile = getPreferencesFile();
        if (preferencesFile.exists()) {
            try {
                InputStream inputStream = new FileInputStream(preferencesFile);
                getPreferences().loadFromXML(inputStream);
            } catch (IOException ex) {
                logger.error("Cannot load preferences", ex);
            }
        }
    }
    
    public static void storePreferences() {
        File preferencesFile = getPreferencesFile();
        File parentDir = preferencesFile.getParentFile();
        if (!parentDir.exists()) parentDir.mkdirs();
        try {
            OutputStream outputStream = new FileOutputStream(preferencesFile);
            getPreferences().storeToXML(outputStream, null);
        } catch (IOException ex) {
            logger.error("Cannot store preferences", ex);
        }
        
    }
    
    private static File getPreferencesFile() {
        String prefDir = System.getenv("APPDATA");
        if (prefDir == null) {
            prefDir = System.getProperty("user.home","");
        }
        return new File(prefDir + File.separatorChar + ".preregcheck.xml");
    }
    
    private static void setPreferredParserProperties() {
        //Creates a new factory instance. The implementation class to load is the first found in the following locations:
        //1. the javax.xml.parsers.SAXParserFactory system property
        //2. the above named property value in the $JAVA_HOME/lib/jaxp.properties file
        //3. the class name specified in the META-INF/services/javax.xml.parsers.SAXParserFactory system resource
        //4. the default factory class
        
        final String SAXPARSERFACTORY = "javax.xml.parsers.SAXParserFactory";
        if (System.getProperty(SAXPARSERFACTORY) == null) {
            System.setProperty(SAXPARSERFACTORY, "org.apache.xerces.jaxp.SAXParserFactoryImpl");
        }
    }
    
}
