/*
 * PreRegistration.java
 *
 * ECHA is providing program source code to facilitate the preparation of
 * pre-registration XML files. The code shall serve as example and
 * registrants are under no circumstances obliged to make use of this service
 * for submitting data to ECHA. In no event shall ECHA be liable for any
 * damages, direct, incidental or consequential (including through negligence
 * or otherwise) which may result from the use of the data provided, even if
 * ECHA has been advised of the possibility of such damages.
 * The users must assume the entire risk of using this source code.
 */

package eu.europa.echa.reach.preregistration;

import eu.europa.echa.reach.preregistration.xml.XMLWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Properties;
import org.apache.log4j.Logger;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 *
 * @author Csk Gbor
 */
public class PreRegistration {
    
    private static final Logger logger = Logger.getLogger(PreRegistration.class);
    
    public static enum Mode {SIMPLE, BULK};
    
    private Properties properties;
    private boolean prepareStatement;
    private Connection connection;
    private BulkPreRegistration bulkPreRegistration = new BulkPreRegistration();
    private List<SinglePreRegistration> singlePreRegistrations = new LinkedList<SinglePreRegistration>();
    
    public PreRegistration(final Properties properties) {
        this.properties = properties;
        String spreparestatement = properties.getProperty("jdbc.driver.preparestatement");
        if (spreparestatement != null) {
            this.prepareStatement = spreparestatement.equalsIgnoreCase("yes");
        }
    }
    
    public void loadSubstanceIdentities(final Connection connection) throws SQLException {
        this.connection = connection;
        
        ResultSet rs = null;
        String sql =  properties.getProperty("database.substanceIdentity.sql");
        if (prepareStatement) {
            PreparedStatement pstmnt = connection.prepareStatement(sql, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
            pstmnt.setBoolean(1, true);
            logger.info("Executing SQL Command: '" + sql + "' (true)");
            rs = pstmnt.executeQuery();
        } else {
            sql = sql.replaceAll("\\{1\\}", "TRUE");
            Statement stmnt = connection.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
            logger.info("Executing SQL Command: \"" + sql + "\"");
            rs = stmnt.executeQuery(sql);
        }
        try {
            while (rs.next()) {
                //handleWarnings(rs.getWarnings());
                
                final SubstanceIdentity substanceIdentity = new SubstanceIdentity();
                String fieldname = null;
                
                fieldname = properties.getProperty("database.substanceIdentity.name");
                if (fieldname != null) substanceIdentity.name = rs.getString(fieldname);
                else logger.info("database.substanceIdentity.name is not defined");
                
                fieldname = properties.getProperty("database.substanceIdentity.enableMessages");
                if (fieldname != null) substanceIdentity.enableMessages = rs.getBoolean(fieldname);
                else logger.info("database.substanceIdentity.enableMessages is not defined");
                
                fieldname = properties.getProperty("database.substanceIdentity.contactPerson");
                if (fieldname != null) substanceIdentity.contactPersonID = rs.getString(fieldname);
                else logger.info("database.substanceIdentity.contactPerson is not defined");
                
                fieldname = properties.getProperty("database.substanceIdentity.thirdPartyRepresentative");
                if (fieldname != null) substanceIdentity.thirdPartyRepresentative = rs.getString(fieldname);
                else logger.info("database.substanceIdentity.thirdPartyRepresentative is not defined");
                
                fieldname = properties.getProperty("database.substanceIdentity.registrationDeadline");
                if (fieldname != null) substanceIdentity.registrationDeadline = rs.getString(fieldname);
                else logger.warn("database.substanceIdentity.registrationDeadline is not defined");
                
                fieldname = properties.getProperty("database.substanceIdentity.tonnageBand");
                if (fieldname != null) substanceIdentity.tonnageBand = rs.getString(fieldname);
                else logger.warn("database.substanceIdentity.tonnageBand is not defined");
                
                fieldname = properties.getProperty("database.substanceIdentity.information");
                if (fieldname != null) substanceIdentity.information = rs.getString("information");
                else logger.info("database.substanceIdentity.information is not defined");
                
                loadConstituents(rs.getString("id"), substanceIdentity.constituents);
                
                boolean isBulk = true;
                fieldname = properties.getProperty("database.substanceIdentity.bulk");
                if (fieldname != null) isBulk = rs.getBoolean(fieldname);
                
                if (isBulk) {
                    bulkPreRegistration.substanceIdentities.add(substanceIdentity);
                    if (substanceIdentity.contactPersonID != null
                            && !bulkPreRegistration.contactPersons.containsKey(substanceIdentity.contactPersonID)) {
                        ContactPerson contactPerson = loadContactPerson(substanceIdentity.contactPersonID);
                        bulkPreRegistration.contactPersons.put(contactPerson.id, contactPerson);
                    }
                } else {
                    SinglePreRegistration singlePreRegistration = new SinglePreRegistration();
                    singlePreRegistration.substanceIdentity = substanceIdentity;
                    if (substanceIdentity.contactPersonID != null) {
                        singlePreRegistration.contactPerson = loadContactPerson(substanceIdentity.contactPersonID);
                    }
                    singlePreRegistrations.add(singlePreRegistration);
                }
            }
        } finally {
            rs.close();
        }
    }
    
    public ContactPerson loadContactPerson(final String id) throws SQLException {
        ContactPerson contactPerson = null;
        
        ResultSet rs = null;
        String sql =  properties.getProperty("database.contactPerson.sql");
        if (prepareStatement) {
            PreparedStatement pstmnt = connection.prepareStatement(sql, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
            pstmnt.setString(1, id);
            logger.info("Executing SQL Command: '" + sql + "' (" + id + ")");
            rs = pstmnt.executeQuery();
        } else {
            sql = sql.replaceAll("\\{1\\}", id);
            Statement stmnt = connection.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
            logger.info("Executing SQL Command: \"" + sql + "\"");
            rs = stmnt.executeQuery(sql);
        }
        try {
            if (rs.next()) {
                //handleWarnings(rs.getWarnings());
                contactPerson = new ContactPerson();
                String fieldname = null;
                
                fieldname = properties.getProperty("database.contactPerson.id");
                if (fieldname != null) contactPerson.id = rs.getString(fieldname);
                else logger.info("database.contactPerson.id is not defined");
                
                fieldname = properties.getProperty("database.contactPerson.title");
                if (fieldname != null) contactPerson.title = rs.getString(fieldname);
                else logger.info("database.contactPerson.title is not defined");
                
                fieldname = properties.getProperty("database.contactPerson.firstName");
                if (fieldname != null) contactPerson.firstName = rs.getString(fieldname);
                else logger.info("database.contactPerson.firstName is not defined");
                
                fieldname = properties.getProperty("database.contactPerson.lastName");
                if (fieldname != null) contactPerson.lastName = rs.getString(fieldname);
                else logger.info("database.contactPerson.lastName is not defined");
                
                fieldname = properties.getProperty("database.contactPerson.remarks");
                if (fieldname != null) contactPerson.remarks = rs.getString(fieldname);
                else logger.info("database.contactPerson.remarks is not defined");
                
                String organisationID = null;
                fieldname = properties.getProperty("database.contactPerson.organisation");
                if (fieldname != null) organisationID = rs.getString(fieldname);
                else logger.info("database.contactPerson.organisation is not defined");
                if (organisationID != null) {
                    contactPerson.organisation = loadOrganisation(organisationID);
                }
                
                String contactInfoID = null;
                fieldname = properties.getProperty("database.contactPerson.contactInfo");
                if (fieldname != null) contactInfoID = rs.getString(fieldname);
                else logger.info("database.contactPerson.contactInfo is not defined");
                if (contactInfoID != null) {
                    contactPerson.contactInfo = loadContactInfo(contactInfoID);
                }
                
                if (rs.next()) {
                    String message = "SQL command '" + sql + "' returned more than one row where expected only one";
                    logger.error(message);
                    throw new SQLException(message);
                }
            } else {
                logger.error("Contactperson id=" + id + " not found");
            }
        } finally {
            rs.close();
        }
        
        return contactPerson;
    }
    
    public Organisation loadOrganisation(final String id) throws SQLException {
        Organisation organisation = null;
        
        ResultSet rs = null;
        String sql =  properties.getProperty("database.organisation.sql");
        if (prepareStatement) {
            PreparedStatement pstmnt = connection.prepareStatement(sql, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
            pstmnt.setString(1, id);
            logger.info("Executing SQL Command: '" + sql + "' (" + id + ")");
            rs = pstmnt.executeQuery();
        } else {
            sql = sql.replaceAll("\\{1\\}", id);
            Statement stmnt = connection.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
            logger.info("Executing SQL Command: \"" + sql + "\"");
            rs = stmnt.executeQuery(sql);
        }
        try {
            if (rs.next()) {
                //handleWarnings(rs.getWarnings());
                organisation = new Organisation();
                String fieldname = null;
                
                fieldname = properties.getProperty("database.organisation.departmentName");
                if (fieldname != null) organisation.departmentName = rs.getString(fieldname);
                else logger.info("database.organisation.departmentName is not defined");
                
                fieldname = properties.getProperty("database.organisation.organisationName");
                if (fieldname != null) organisation.organisationName = rs.getString(fieldname);
                else logger.info("database.organisation.organisationName is not defined");
                
                String contactInfoID = null;
                fieldname = properties.getProperty("database.organisation.contactInfo");
                if (fieldname != null) contactInfoID = rs.getString(fieldname);
                else logger.info("database.organisation.contactInfo is not defined");
                
                if (contactInfoID != null) {
                    organisation.contactInfo = loadContactInfo(contactInfoID);
                }
                
                if (rs.next()) {
                    String message = "SQL command '" + sql + "' returned more than one row";
                    logger.error(message);
                    throw new SQLException(message);
                }
            } else {
                logger.error("Organisation id=" + id + " not found");
            }
        } finally {
            rs.close();
        }
        return organisation;
    }
    
    public ContactInfo loadContactInfo(final String id) throws SQLException {
        ContactInfo contactInfo = null;
        
        ResultSet rs = null;
        String sql =  properties.getProperty("database.contactInfo.sql");
        if (prepareStatement) {
            PreparedStatement pstmnt = connection.prepareStatement(sql, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
            pstmnt.setString(1, id);
            logger.info("Executing SQL Command: '" + sql + "' (" + id + ")");
            rs = pstmnt.executeQuery();
        } else {
            sql = sql.replaceAll("\\{1\\}", id);
            Statement stmnt = connection.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
            logger.info("Executing SQL Command: \"" + sql + "\"");
            rs = stmnt.executeQuery(sql);
        }
        try {
            if (rs.next()) {
                //handleWarnings(rs.getWarnings());
                contactInfo = new ContactInfo();
                String fieldname = null;
                
                fieldname = properties.getProperty("database.contactInfo.phone");
                if (fieldname != null) contactInfo.phone = rs.getString(fieldname);
                else logger.warn("database.contactInfo.phone is not defined");
                
                fieldname = properties.getProperty("database.contactInfo.fax");
                if (fieldname != null) contactInfo.fax = rs.getString(fieldname);
                else logger.info("database.contactInfo.fax is not defined");
                
                fieldname = properties.getProperty("database.contactInfo.mobile");
                if (fieldname != null) contactInfo.mobile = rs.getString(fieldname);
                else logger.info("database.contactInfo.mobile is not defined");
                
                fieldname = properties.getProperty("database.contactInfo.email");
                if (fieldname != null) contactInfo.email = rs.getString(fieldname);
                else logger.info("database.contactInfo.email is not defined");
                
                fieldname = properties.getProperty("database.contactInfo.website");
                if (fieldname != null) contactInfo.webSite = rs.getString(fieldname);
                else logger.info("database.contactInfo.website is not defined");
                
                {
                    PostalAddress postalAddress = new PostalAddress();
                    
                    fieldname = properties.getProperty("database.contactInfo.postalAddress.street1");
                    if (fieldname != null) postalAddress.street1 = rs.getString(fieldname);
                    else logger.warn("database.contactInfo.postalAddress.street1 is not defined");
                    
                    fieldname = properties.getProperty("database.contactInfo.postalAddress.street2");
                    if (fieldname != null) postalAddress.street2 = rs.getString(fieldname);
                    else logger.info(" is not defined");
                    
                    fieldname = properties.getProperty("database.contactInfo.postalAddress.zipCode");
                    if (fieldname != null) postalAddress.zipCode = rs.getString(fieldname);
                    else logger.warn("database.contactInfo.postalAddress.zipCode is not defined");
                    
                    fieldname = properties.getProperty("database.contactInfo.postalAddress.city");
                    if (fieldname != null) postalAddress.city = rs.getString(fieldname);
                    else logger.info("database.contactInfo.postalAddress.city is not defined");
                    
                    fieldname = properties.getProperty("database.contactInfo.postalAddress.state");
                    if (fieldname != null) postalAddress.state = rs.getString(fieldname);
                    else logger.info("database.contactInfo.postalAddress.state is not defined");
                    
                    fieldname = properties.getProperty("database.contactInfo.postalAddress.country");
                    if (fieldname != null) postalAddress.country = rs.getString(fieldname);
                    else logger.warn("database.contactInfo.postalAddress.country is not defined");
                    
                    fieldname = properties.getProperty("database.contactInfo.postalAddress.envelopeAddress");
                    if (fieldname != null) postalAddress.envelopeAddress = rs.getString(fieldname);
                    
                    if (
                            postalAddress.street1 != null ||
                            postalAddress.street2 != null ||
                            postalAddress.zipCode != null ||
                            postalAddress.city != null ||
                            postalAddress.state != null ||
                            postalAddress.country != null ||
                            postalAddress.envelopeAddress != null) {
                        contactInfo.postalAddress = postalAddress;
                    }
                }
                if (rs.next()) {
                    String message = "SQL command '" + sql + "' returned more than one row where expected only one";
                    logger.error(message);
                    throw new SQLException(message);
                }
            } else {
                logger.error("ContactInfo id=" + id + " not found");
            }
        } finally {
            rs.close();
        }
        return contactInfo;
    }
    
    private void loadConstituents(final String substanceID, final List<SubstanceConstituentIdentity> constituents) throws SQLException {
        
        ResultSet rs = null;
        String sql =  properties.getProperty("database.constituent.sql");
        if (prepareStatement) {
            PreparedStatement pstmnt = connection.prepareStatement(sql, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
            pstmnt.setString(1, substanceID);
            logger.info("Executing SQL Command: '" + sql + "' (" + substanceID + ")");
            rs = pstmnt.executeQuery();
        } else {
            sql = sql.replaceAll("\\{1\\}", substanceID);
            Statement stmnt = connection.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
            logger.info("Executing SQL Command: \"" + sql + "\"");
            rs = stmnt.executeQuery(sql);
        }
        try {
            while (rs.next()) {
                //handleWarnings(rs.getWarnings());
                SubstanceConstituentIdentity constituent = new SubstanceConstituentIdentity();
                String fieldname = null;
                
                fieldname = properties.getProperty("database.constituent.ecNumber");
                if (fieldname != null) constituent.ecNumber = rs.getString(fieldname);
                else logger.warn("database.constituent.ecNumber is not defined");
                
                fieldname = properties.getProperty("database.constituent.casNumber");
                if (fieldname != null) constituent.casNumber = rs.getString(fieldname);
                else logger.info("database.constituent.casNumber is not defined");
                
                fieldname = properties.getProperty("database.constituent.ecName");
                if (fieldname != null) constituent.ecName = rs.getString(fieldname);
                else logger.info("database.constituent.ecName is not defined");
                
                fieldname = properties.getProperty("database.constituent.casName");
                if (fieldname != null) constituent.casName = rs.getString(fieldname);
                else logger.info("database.constituent.casName is not defined");
                
                fieldname = properties.getProperty("database.constituent.iupacName");
                if (fieldname != null) constituent.iupacName = rs.getString(fieldname);
                else logger.info("database.constituent.iupacName is not defined");
                
                fieldname = properties.getProperty("database.constituent.otherName");
                if (fieldname != null) constituent.otherName = rs.getString(fieldname);
                else logger.info("database.constituent.otherName is not defined");
                
                fieldname = properties.getProperty("database.constituent.lang");
                if (fieldname != null) constituent.lang = rs.getString(fieldname);
                else logger.info("database.constituent.lang is not defined");
                
                constituents.add(constituent);
            }
        } finally {
            rs.close();
        }
    }
    
    public void serialize(final String fileName) throws SerializationException {
        bulkPreRegistration.serialize(fileName);
        ListIterator<SinglePreRegistration> iterator = singlePreRegistrations.listIterator();
        while (iterator.hasNext()) {
            iterator.next().serialize(fileName);
        }
    }
    
    private void handleWarnings(SQLWarning warning) {
        if (warning != null) {
            logger.warn("SQL warning: " + warning.getMessage() + " code: " + warning.getErrorCode());
            // handle linked warnings
            handleWarnings(warning.getNextWarning());
        }
    }
    
    private class BulkPreRegistration {
        List<SubstanceIdentity> substanceIdentities = new LinkedList<SubstanceIdentity>();
        Map<String,ContactPerson> contactPersons = new LinkedHashMap<String,ContactPerson>();
        
        public void serialize(final String fileName) throws SerializationException {
            try {
                File file = new File(fileName + ".xml");
                if (file.exists()) {
                    logger.warn("Overwriting file: " + file.getAbsolutePath());
                }
                OutputStream outputStream = new FileOutputStream(file);
                XMLWriter xmlWriter = new XMLWriter(outputStream);
                
                {
                    final String BULKPREREGISTRATION = "BulkPreRegistration";
                    {
                        AttributesImpl attributes = new AttributesImpl();
                        attributes.addAttribute(XMLWriter.NS.PRG.getNamespaceUri(), "schemaVersion", XMLWriter.NS.PRG.getPrefix() + ":schemaVersion", "", "4.0");
                        xmlWriter.startElement(XMLWriter.NS.PRG, BULKPREREGISTRATION, attributes);
                    }
                    if (contactPersons.size() > 0) {
                        final String CONTACTPERSONS = "ContactPersons";
                        xmlWriter.startElement(XMLWriter.NS.PRG, CONTACTPERSONS);
                        Iterator<ContactPerson> cpIterator = contactPersons.values().iterator();
                        while (cpIterator.hasNext()) {
                            cpIterator.next().serialize(xmlWriter, Mode.BULK);
                        }
                        xmlWriter.endElement(XMLWriter.NS.PRG, CONTACTPERSONS);
                    }
                    {
                        final String PREREGISTRATIONS = "PreRegistrations";
                        xmlWriter.startElement(XMLWriter.NS.PRG, PREREGISTRATIONS);
                        ListIterator<SubstanceIdentity> substIterator = substanceIdentities.listIterator();
                        while (substIterator.hasNext()) {
                            SubstanceIdentity substanceIdentity = substIterator.next();
                            final String PREREGISTRATION = "PreRegistration";
                            {
                                AttributesImpl attributes = new AttributesImpl();
                                attributes.addAttribute("","","enableMessages", "", substanceIdentity.enableMessages? "1" : "0");
                                xmlWriter.startElement(XMLWriter.NS.PRG, PREREGISTRATION, attributes);
                            }
                            substanceIdentity.serialize(xmlWriter, Mode.BULK);
                            //contctPerson
                            if (substanceIdentity.contactPersonID != null) {
                                AttributesImpl attributes = new AttributesImpl();
                                attributes.addAttribute("","","idref","", substanceIdentity.contactPersonID);
                                xmlWriter.writeElement(XMLWriter.NS.PRG, "contactPerson", attributes, null);
                            }
                            //thirdPartyRepresentative
                            if (substanceIdentity.thirdPartyRepresentative != null) {
                                AttributesImpl attributes = new AttributesImpl();
                                attributes.addAttribute("","","partyUUID","", substanceIdentity.thirdPartyRepresentative);
                                xmlWriter.writeElement(XMLWriter.NS.PRG, "thirdPartyRepresentative", attributes, null);
                            }
                            //registrationDeadline
                            xmlWriter.writeElement(XMLWriter.NS.PRG, "registrationDeadline", substanceIdentity.registrationDeadline);
                            //tonnageBand
                            xmlWriter.writeElement(XMLWriter.NS.PRG, "tonnageBand", substanceIdentity.tonnageBand);
                            //similarSubstanceIdentities //Not Implemented
                            //information
                            if (substanceIdentity.information != null) {
                                final String INFORATION = "information";
                                xmlWriter.writeElement(XMLWriter.NS.PRG, INFORATION, substanceIdentity.information);
                            }
                            xmlWriter.endElement(XMLWriter.NS.PRG, PREREGISTRATION);
                        }
                        xmlWriter.endElement(XMLWriter.NS.PRG, PREREGISTRATIONS);
                    }
                    xmlWriter.endElement(XMLWriter.NS.PRG, BULKPREREGISTRATION);
                }
                xmlWriter.endDocument();
                outputStream.close();
            } catch (IOException ex) {
                throw new SerializationException(ex);
            } catch (SAXException ex) {
                throw new SerializationException(ex);
            }
        }
    }
    
    private class SinglePreRegistration {
        ContactPerson contactPerson;
        SubstanceIdentity substanceIdentity;
        public void serialize(final String fileName) throws SerializationException {
            try {
                String suffix = null;
                ListIterator<SubstanceConstituentIdentity> iterator = substanceIdentity.constituents.listIterator();
                while (iterator.hasNext()) {
                    SubstanceConstituentIdentity constituent = iterator.next();
                    if (constituent.casNumber != null) {
                        if (suffix == null) suffix = "";
                        suffix += "_" + constituent.casNumber;
                    }
                }
                if (suffix == null) {
                    Date now = new Date();
                    SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd_HH-mm-ss");
                    suffix = "_" + df.format(now);
                }
                File file = new File(fileName + suffix + ".xml");
                if (file.exists()) {
                    logger.warn("Overwriting file: " + file.getAbsolutePath());
                }
                OutputStream outputStream = new FileOutputStream(file);
                XMLWriter xmlWriter = new XMLWriter(outputStream);
                
                {
                    final String SIMPLEPREREGISTRATION = "SimplePreRegistration";
                    {
                        AttributesImpl attributes = new AttributesImpl();
                        attributes.addAttribute(XMLWriter.NS.PRG.getNamespaceUri(), "schemaVersion", XMLWriter.NS.PRG.getPrefix() + ":schemaVersion", "", "4.0");
                        xmlWriter.startElement(XMLWriter.NS.PRG, SIMPLEPREREGISTRATION, attributes);
                    }
                    {
                        substanceIdentity.serialize(xmlWriter, Mode.SIMPLE);
                        if (contactPerson != null) {
                            contactPerson.serialize(xmlWriter, Mode.SIMPLE);
                        }
                        //thirdPartyRepresentative
                        if (substanceIdentity.thirdPartyRepresentative != null) {
                            AttributesImpl attributes = new AttributesImpl();
                            attributes.addAttribute("","","partyUUID","", substanceIdentity.thirdPartyRepresentative);
                            xmlWriter.writeElement(XMLWriter.NS.PRG, "thirdPartyRepresentative", attributes, null);
                        }
                        //registrationDeadline
                        xmlWriter.writeElement(XMLWriter.NS.PRG, "registrationDeadline", substanceIdentity.registrationDeadline);
                        //tonnageBand
                        xmlWriter.writeElement(XMLWriter.NS.PRG, "tonnageBand", substanceIdentity.tonnageBand);
                        //similarSubstanceIdentities //Not Implemented
                        //information
                        if (substanceIdentity.information != null) {
                            final String INFORATION = "information";
                            xmlWriter.writeElement(XMLWriter.NS.PRG, INFORATION, substanceIdentity.information);
                        }
                    }
                    xmlWriter.endElement(XMLWriter.NS.PRG, SIMPLEPREREGISTRATION);
                }
                xmlWriter.endDocument();
                outputStream.close();
            } catch (IOException ex) {
                throw new SerializationException(ex);
            } catch (SAXException ex) {
                throw new SerializationException(ex);
            }
        }
    }
}
