/*
 * XMLWriter.java
 *
 * ECHA is providing program source code to facilitate the preparation of
 * pre-registration XML files. The code shall serve as example and
 * registrants are under no circumstances obliged to make use of this service
 * for submitting data to ECHA. In no event shall ECHA be liable for any
 * damages, direct, incidental or consequential (including through negligence
 * or otherwise) which may result from the use of the data provided, even if
 * ECHA has been advised of the possibility of such damages.
 * The users must assume the entire risk of using this source code.
 */

package eu.europa.echa.reach.preregistration.xml;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Properties;
import javax.xml.transform.OutputKeys;
import org.apache.xml.serializer.OutputPropertiesFactory;
import org.apache.xml.serializer.Serializer;
import org.apache.xml.serializer.SerializerFactory;
import org.apache.xml.serializer.ToStream;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 *
 * @author Csk Gbor
 */
public class XMLWriter {
    
    public enum NS {
        PRG("prg", "http://echa.europa.eu/schemas/preRegistration"),
        CTI("cti","http://echa.europa.eu/schemas/contactInfo"),
        SID("sid","http://echa.europa.eu/schemas/substanceIdentity");
        
        private String prefix;
        private String namespaceUri;
        NS(String prefix, String namespaceUri) {
            this.prefix = prefix;
            this.namespaceUri = namespaceUri;
        }
        public String getPrefix() {
            return prefix;
        }
        public String getNamespaceUri() {
            return namespaceUri;
        }
    }
    
    private ContentHandler ch;
    
    public XMLWriter(OutputStream outputStream) throws IOException, SAXException{
        //http://www.jdocs.com/xalan/2.7.0/org/apache/xml/serializer/SerializerFactory.html
        Properties props = OutputPropertiesFactory.getDefaultMethodProperties("xml");
        // The default encoding is utf-8 but setting it explicitely does not hurt
        props.setProperty(OutputKeys.ENCODING, "UTF-8");
        // Set "no" if you do not want pretty printed xml
        props.setProperty(OutputKeys.INDENT, "yes");
        props.setProperty(OutputPropertiesFactory.S_KEY_INDENT_AMOUNT, "3");
        //props.list(System.out);
        
        Serializer s = SerializerFactory.getSerializer(props);
        s.setOutputStream(outputStream);
        ch  = s.asContentHandler();
        ch.startDocument();
        
        ch.startPrefixMapping(NS.PRG.prefix, NS.PRG.namespaceUri);
        ch.startPrefixMapping(NS.CTI.prefix, NS.CTI.namespaceUri);
        ch.startPrefixMapping(NS.SID.prefix, NS.SID.namespaceUri);
    }
    
    public void startElement(NS ns, String localName, Attributes attributes) throws SAXException {
        String qName = ns.prefix + ":" + localName;
        ch.startElement(ns.namespaceUri, localName, qName, attributes);
    }
    
    public void startElement(NS ns, String localName) throws SAXException {
        Attributes atts = new AttributesImpl();
        startElement(ns, localName, atts);
    }
    
    public void endElement(NS ns, String localName) throws SAXException {
        String qName = ns.prefix + ":" + localName;
        ch.endElement(ns.namespaceUri, localName, qName);
    }
    
    public void writeElement(NS ns, String localName, Attributes attributes, String text) throws SAXException {
        String qName = ns.prefix + ":" + localName;
        ch.startElement(ns.namespaceUri, localName, qName, attributes);
        if (text != null && text.length() > 0) {
            ch.characters(text.toCharArray(), 0, text.length());
        }
        ch.endElement(ns.namespaceUri, localName, qName);
    }
    
    public void writeElement(NS ns, String localName, String text) throws SAXException {
        writeElement(ns, localName, null, text);
    }
    
    public void writeText(String text) throws SAXException {
        if (text != null && text.length() > 0) {
            ch.characters(text.toCharArray(), 0, text.length());
        }
    }
    
    public void writeCDATA(String text) throws SAXException {
        if (text != null && text.length() > 0) {
            ToStream toStream = (ToStream)ch;
            toStream.startCDATA();
            toStream.characters(text.toCharArray(), 0, text.length());
            toStream.endCDATA();
        }
    }

    public void writeComment(String comment) throws SAXException {
        if (comment != null) {
            ToStream toStream = (ToStream)ch;
            toStream.comment(comment);
        }
    }
    
    public void endDocument() throws SAXException {
        ch.endDocument();
    }
}
